# Overview

This charm provides a partial caching proxy mirror of the 
[Ubuntu Software Repository](https://help.ubuntu.com/community/Repositories/Ubuntu
"Repository Documentation").  This is intended for deployment in cloud 
environments to provide a cloud-local repository.

This is a hybrid mirror / cache.  Repository metadata, data under the
ubuntu/dists/ directory, is copied from upstream Ubuntu mirrors and
checked to ensure that it is consistent.  Requests for package files in
/ubuntu/pool are forwarded internally to squid-deb-proxy, which keeps a
local cache of .deb files as they are requested.  This approach minimized
load on the upstream archive server, improves performance, and requires
less disk space than a static archive mirror.

# Usage

Deploy the charm with these example commands:

    # Create cache units
    juju set-constraints --service ubuntu-repository-cache mem=8G root-disk=80G
    juju deploy ubuntu-repository-cache

    # Provide ntp service with local NTP masters
    juju deploy ntp
    juju deploy -n 2 ntpmaster
    juju add-relationship ntp ntpmaster
    juju add-relationship ntp ubuntu-repository-cache
    
The ubuntu-repository-cache charm's constraint for disk size is optional,
the intention is to allocate sufficient space for the metadata mirror
(approximately 4GB) plus as much space as can be afforded for squid to
cache package files.

Ideally this charm should be deployed with this disk space allocated
as fast storage (which may require ephemeral storage depending on the
provider); use of the manual provider may be necessary to achieve this.

When running, you can browse to http://ip-address to view the repository
when the initial mirror sync has completed.  Alternatively, you may add
a proxy in front of this service.

## Scale out usage

When additional units are added the content they serve will be
synchronized from the lead unit.  As the service is scaled, the use of
haproxy in front of the mirror may be desirable to distribute load.

## Disabling OS updates/upgrades

When deploying new units of this charm, Juju may update the operating
system.  This charm could fail to deploy if this charm is deployed as
mirror service for the package repository configured in the cloud image.

If that is the case, the juju environment configuration should be edited
to disable OS update/upgrade.  The charm will change the apt source to
point at the archive specified in the 'sync-host' charm configuration
and will perform an update during charm installation.
(See https://bugs.launchpad.net/juju-core/+bug/1350493/comments/4)

## Known Limitations and Issues

Beta quality

* See the 'BUGS' file or [Launchpad](https://bugs.launchpad.net/charms/+source/ubuntu-repository-cache "ubuntu-repository-cache bugs in Launchpad")

* When the service is destroyed and multiple units are deployed, once the
  leader (the lowest number unit) departs, the next lowest will sync
  from the master archive to become the new leader as it is unaware that
  the entire service is being destroyed.  Remove all but the lowest numbered
  unit from the service and then destroy the service; this can be achieved by
  reducing the number of units for the service to 1.

# Configuration

* sync-host - The host name or IP address of the archive which will
  be used to keep this mirror updated.  The mirror must support 'rsync'
  access.

* use-ephemeral - When set, the charm will use the file-system mounted
  at the location specified by the 'ephemeral-mount' option for the
  metadata mirror and the package cache.  This setting can not be changed
  at runtime.

* ephemeral-mount - The file-system location where mirrored metadata and 
  cached packages should be stored.  This value is used when the
  'use-ephemeral' option is specified and can not be changed at runtime.

# Contact Information

Questions and comments can be posted to ubuntu-cloud@lists.ubuntu.com, see 
https://lists.ubuntu.com/mailman/listinfo/ubuntu-cloud to subscribe to this
mailing list.

Bugs can be viewed or reported at https://bugs.launchpad.net/charms/+source/ubuntu-repository-cache
