'''Functions related to apache use within this charm'''

import os
import subprocess

# pylint can't find the modules  # pylint: disable=F0401
from charmhelpers import (
    fetch,
)

from charmhelpers.core import (
    hookenv,
    host,
    templating
)
import util
# pylint: enable=F0401


LOG = hookenv.log
SERVICE = 'apache2'


@util.run_as_user('root')
def install():
    '''Perform the install/config of apache and disable the default site'''

    fetch.apt_install(SERVICE, fatal=True)
    stop()
    subprocess.check_call(['a2enmod', 'expires'])
    subprocess.check_call(['a2enmod', 'proxy'])
    subprocess.check_call(['a2enmod', 'proxy_http'])
    subprocess.check_call(['a2enmod', 'headers'])
    subprocess.check_call(['a2dissite', '000-default'])


@util.run_as_user('root')
def start():
    '''Start the service and remove the override file blocking startup'''

    override_path = '/etc/init/apache2.override'
    if os.path.exists(override_path):
        os.remove('/etc/init/apache2.override')

    subprocess.call(['a2ensite', 'archive_ubuntu_com'])
    host.service_start(SERVICE)


@util.run_as_user('root')
def stop():
    '''Stop the service and block startup at boot'''

    with open('/etc/init/apache2.override', 'w+') as override_file:
        override_file.write('manual\n')

    host.service_stop(SERVICE)
    subprocess.call(['a2dissite', 'archive_ubuntu_com'])


@util.run_as_user('root')
def service_running():
    '''Return True is the service is running'''

    return host.service_running(SERVICE)


@util.run_as_user('root')
def render_configs():
    '''Render the configuration templates for apache'''

    LOG('Rendering apache2 configuration templates')
    config = hookenv.config()
    apache_context = {}
    apache_context['DocumentRoot'] = config['apache-root']
    apache_context['Sync_Host'] = config['sync-host']
    templating.render('apache2/archive_ubuntu_com.conf',
                      '/etc/apache2/sites-available/archive_ubuntu_com.conf',
                      apache_context)
